@@ -15,6 +15,7 @@
 """Base classes and functions for C/C++ linters and formatters."""
 
 import argparse
+import json
 import logging
 import os
 import platform
@@ -82,6 +83,16 @@ def get_cmake_command(cmake_names=None):
     return None
 
 
+def _read_compile_commands_json(build_dir: Path):
+    """Read a JSON compile database and return the list of files contained within."""
+    compile_db = build_dir / 'compile_commands.json'
+    if compile_db.exists():
+        with open(str(compile_db), encoding='utf-8') as fd:
+            data = json.load(fd)
+        return [entry['file'] for entry in data]
+    return []
+
+
 def _append_in_namespace(namespace, key, values):
     current = getattr(namespace, key, [])
     if current is None:
@@ -145,6 +156,7 @@ class Command(hooks.utils.Command):  # pylint: disable=too-many-instance-attribu
         self.build_dir = '.cmake_build'
         self.cmake_args = ['-DCMAKE_EXPORT_COMPILE_COMMANDS=ON']
         self.all_at_once = False
+        self.read_json_db = False
 
         self.history = []
 
@@ -161,17 +173,29 @@ class Command(hooks.utils.Command):  # pylint: disable=too-many-instance-attribu
         parser.add_argument('-S', '--source-dir', type=str, help='Path to build directory', default=self.source_dir)
         parser.add_argument('-B', '--build-dir', action='append', type=str, help='Path to build directory')
         parser.add_argument(
-            '-D', dest='defines', action='append', type=str, help='Create or update a cmake cache entry.'
+            '-D', dest='defines', action='append', type=str, help='Create or update a cmake cache entry.', default=[]
         )
         parser.add_argument(
-            '-U', dest='undefines', action='append', type=str, help='Remove matching entries from CMake cache.'
+            '-U',
+            dest='undefines',
+            action='append',
+            type=str,
+            help='Remove matching entries from CMake cache.',
+            default=[],
         )
         parser.add_argument('-G', dest='generator', type=str, help='Specify a build system generator.')
         parser.add_argument('-T', dest='toolset', type=str, help='Specify toolset name if supported by generator.')
         parser.add_argument('-A', dest='platform', type=str, help='Specify platform if supported by generator.')
-        parser.add_argument('-Werror', dest='errors', action='append', type=str, help='Make developer warnings errors.')
         parser.add_argument(
-            '-Wno-error', dest='no_error', action='append', type=str, help='Make developer warnings not errors.'
+            '-Werror', dest='errors', action='append', type=str, help='Make developer warnings errors.', default=[]
+        )
+        parser.add_argument(
+            '-Wno-error',
+            dest='no_error',
+            action='append',
+            type=str,
+            help='Make developer warnings not errors.',
+            default=[],
         )
         parser.add_argument(
             '--cmake', type=executable_path, help='Specify path to CMake executable.', default=self.cmake
@@ -185,6 +209,14 @@ class Command(hooks.utils.Command):  # pylint: disable=too-many-instance-attribu
             action='store_true',
             help='Pass all filenames at once to the linter/formatter instead of calling the command once for each file',
         )
+        parser.add_argument(
+            '--read-json-db',
+            action='store_true',
+            help=(
+                'Run hooks on files found in compile_commands.json '
+                '(if found and in addition to files specified on CLI)'
+            ),
+        )
 
         parser.add_argument(
             '--unix',
@@ -207,6 +239,7 @@ class Command(hooks.utils.Command):  # pylint: disable=too-many-instance-attribu
 
         self.cmake = known_args.cmake
         self.all_at_once = known_args.all_at_once
+        self.read_json_db = known_args.read_json_db
         self.clean_build = known_args.clean
         self.source_dir = Path(known_args.source_dir).resolve()
 
@@ -237,6 +270,8 @@ class Command(hooks.utils.Command):  # pylint: disable=too-many-instance-attribu
     def run(self):
         """Run the command."""
         self.run_cmake_configure()
+        if self.read_json_db:
+            self.files.extend(set(self.files).symmetric_difference(set(_read_compile_commands_json(self.build_dir))))
         if self.all_at_once:
             self.run_command(self.files)
         elif self.files:
@@ -258,17 +293,20 @@ class Command(hooks.utils.Command):  # pylint: disable=too-many-instance-attribu
         """Run the command and check for errors."""
         self.history.append(self._call_process([self.command] + filenames + self.args + self.ddash_args))
 
+        # Compatibility with CLinters
+        self.stdout = self.history[-1].stdout.encode()
+        self.stderr = self.history[-1].stderr.encode()
+        self.returncode = self.history[-1].returncode
+
     def run_cmake_configure(self):  # pylint: disable=too-many-branches
         """Run a CMake configure step."""
         configuring = Path(self.build_dir, '_configuring')
-        has_lock = False
 
         self.build_dir.mkdir(exist_ok=True)
 
         rw_lock = fasteners.InterProcessReaderWriterLock(configuring)
         if not configuring.exists():
             with rw_lock.write_lock():
-                has_lock = True
                 if self.clean_build:
                     for path in self.build_dir.iterdir():
                         if path.is_dir():
@@ -312,14 +350,12 @@ class Command(hooks.utils.Command):  # pylint: disable=too-many-instance-attribu
 
                 self.history.append(result)
                 returncode = result.returncode
+            configuring.unlink()
         else:
             returncode = 0
             with rw_lock.read_lock():
                 pass
 
-        if has_lock and configuring.exists():
-            configuring.unlink()
-
         if returncode != 0:
             sys.exit(returncode)
 
@@ -346,15 +382,14 @@ class Command(hooks.utils.Command):  # pylint: disable=too-many-instance-attribu
         if not self.cmake:
             raise RuntimeError('Unable to locate CMake command')
 
-        if args.defines:
-            for define in args.defines:
-                self.cmake_args.append(f'-D{define}')
-        if args.undefines:
-            for undefine in args.undefines:
-                self.cmake_args.append(f'-U{undefine}')
-        if args.errors:
-            for error in args.errors:
-                self.cmake_args.append(f'-Werror={error}')
+        for define in args.defines:
+            self.cmake_args.append(f'-D{define}')
+
+        for undefine in args.undefines:
+            self.cmake_args.append(f'-U{undefine}')
+
+        for error in args.errors:
+            self.cmake_args.append(f'-Werror={error}')
 
         if args.generator:
             self.cmake_args.append(f'-G{args.generator}')
@@ -421,3 +456,7 @@ class FormatterCmd(Command, hooks.utils.FormatterCmd):
         if child.stdout == "":
             return []
         return child.stdout.encode().split(b"\x0a")
+
+
+class StaticAnalyzerCmd(Command, hooks.utils.StaticAnalyzerCmd):
+    """Commands that analyze code and are not formatters."""
